import { isArray, isNumber } from 'lodash';
import { useEffect, useMemo, useRef, useState } from 'react';
import { useAsync } from '@react-devui/hooks';
export function DTransition(props) {
    const { children, dIn, dDuring, dMountBeforeFirstEnter = true, dSkipFirstTransition = true, dDestroyWhenLeaved = false, onEnter, afterEnter, afterLeave, } = props;
    const initState = useMemo(() => {
        const [skipEnter, skipLeave] = isArray(dSkipFirstTransition) ? dSkipFirstTransition : [dSkipFirstTransition, dSkipFirstTransition];
        if (dIn) {
            return skipEnter ? 'entered' : 'enter';
        }
        else {
            return skipLeave ? 'leaved' : 'leave';
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const dataRef = useRef({
        prevIn: dIn,
        state: initState,
        isFirstEnter: !dIn,
    });
    const async = useAsync();
    const [stateChange, setStateChange] = useState(0);
    if (dataRef.current.prevIn !== dIn) {
        dataRef.current.prevIn = dIn;
        dataRef.current.state = dIn ? 'enter' : 'leave';
        dataRef.current.isFirstEnter = false;
    }
    const state = dataRef.current.state;
    const nextFrame = (cb) => {
        dataRef.current.clearTid = async.requestAnimationFrame(() => {
            dataRef.current.clearTid = async.setTimeout(cb);
        });
    };
    useEffect(() => {
        if (state === 'entered') {
            onEnter === null || onEnter === void 0 ? void 0 : onEnter();
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    useEffect(() => {
        var _a, _b;
        (_b = (_a = dataRef.current).clearTid) === null || _b === void 0 ? void 0 : _b.call(_a);
        if (state === 'enter') {
            onEnter === null || onEnter === void 0 ? void 0 : onEnter();
            nextFrame(() => {
                dataRef.current.state = 'entering';
                setStateChange((prevStateChange) => prevStateChange + 1);
            });
        }
        else if (state === 'leave') {
            nextFrame(() => {
                dataRef.current.state = 'leaving';
                setStateChange((prevStateChange) => prevStateChange + 1);
            });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [dIn]);
    useEffect(() => {
        if (dataRef.current.state === 'entering' || dataRef.current.state === 'leaving') {
            const during = isNumber(dDuring) ? dDuring : dIn ? dDuring.enter : dDuring.leave;
            dataRef.current.clearTid = async.setTimeout(() => {
                dataRef.current.state = dIn ? 'entered' : 'leaved';
                setStateChange((prevStateChange) => prevStateChange + 1);
            }, during);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [stateChange]);
    useEffect(() => {
        if (dataRef.current.state === 'entered' || dataRef.current.state === 'leaved') {
            dIn ? afterEnter === null || afterEnter === void 0 ? void 0 : afterEnter() : afterLeave === null || afterLeave === void 0 ? void 0 : afterLeave();
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [stateChange]);
    const shouldRender = (() => {
        if (state === 'leaved' && dDestroyWhenLeaved) {
            return false;
        }
        if (dataRef.current.isFirstEnter && !dMountBeforeFirstEnter) {
            return false;
        }
        return true;
    })();
    return shouldRender ? children(state) : null;
}
